@@ -6,6 +6,9 @@ use warnings;
 use Carp;
 use Config;
 
+use My::Module::Recommend::Any qw{ __any };
+use My::Module::Recommend::All qw{ __all };
+
 my ( $is_5_010, $is_5_012 );
 
 eval {
@@ -19,23 +22,25 @@ eval {
 my %misbehaving_os = map { $_ => 1 } qw{ MSWin32 cygwin };
 
 my @optionals = (
-    [ 'Astro::SIMBAD::Client'	=> <<'EOD' ],
+    __any( 'Astro::SIMBAD::Client'	=> <<'EOD' ),
       This module is required for the 'lookup' subcommand of the
-      Astro::App::Satpass2 sky() method, but is otherwise unused by this
-      package. If you do not intend to use this functionality,
-      Astro::SIMBAD::Client is not needed.
+      Astro::App::Satpass2 sky() method, which retrieves astronomical
+      objects from the SIMBAD database. If you do not intend to use
+      this functionality, Astro::SIMBAD::Client is not needed.
 EOD
-    [ 'Astro::SpaceTrack'	=> <<'EOD' ],
-      This module is required for the Astro::App::Satpass2 st() method,
-      but is otherwise unused by this package. If you do not intend to
-      use this functionality, Astro::SpaceTrack is not needed.
+    __any( 'Astro::SpaceTrack'	=> <<'EOD' ),
+      This module is required for the Astro::App::Satpass2 spacetrack()
+      method, which retrieves satellite TLE data from Space Track and
+      other web sites.  If you intend to get your TLEs externally to
+      this package (say, with a web browser or curl), Astro::SpaceTrack
+      is not needed.
 EOD
-    [ 'Date::Manip'		=> <<'EOD' .
+    __any( 'Date::Manip'		=> <<'EOD' .
       This module is not required, but the alternative to installing it
       is to specify times in ISO 8601 format.  See 'SPECIFYING TIMES' in
       the 'Astro::App::Satpass2' documentation for the details.
 EOD
-	( $is_5_010 ? '' : <<'EOD' ) ],
+	( $is_5_010 ? '' : <<'EOD' ) ),
 
       Unfortunately, the current Date::Manip requires Perl 5.10. Since
       you are running an earlier Perl, you can try installing Date-Manip
@@ -43,24 +48,25 @@ EOD
       Perl 5.10. This version of Date::Manip does not understand summer
       time (a.k.a. daylight saving time).
 EOD
-    [ [ qw{ DateTime DateTime::TimeZone } ] => <<'EOD' ],
+    __all( qw{ DateTime DateTime::TimeZone }	=> <<'EOD' ),
       These modules are used to format times, and provide full time zone
       support. If they are not installed, POSIX::strftime() will be
       used, and you may find that you can not display correct local
-      times for zones other than your system's default zone.
+      times for zones other than your system's default zone or GMT.
 EOD
-    [ 'Geo::Coder::OSM'		=> <<'EOD' ],
+    __any( 'Geo::Coder::OSM'		=> <<'EOD' ),
       This module is required for the Astro::App::Satpass2 geocode()
-      method. If you do not intend to use this functionality, this
-      module is not needed.
+      method, which computes latitude and longitude based on street
+      address. If you do not intend to determine your observing
+      locations this way, this module is not needed.
 EOD
-    [ 'Geo::WebService::Elevation::USGS'	=> <<'EOD' ],
+    __any( 'Geo::WebService::Elevation::USGS'	=> <<'EOD' ),
       This module is required for the Astro::App::Satpass2 height()
-      method, but is otherwise unused by this package. If you do not
-      intend to use this functionality, Geo::WebService::Elevation::USGS
-      is not needed.
+      method, which determines height above the reference ellipsoid for
+      a given latitude and longitude.  If you do not intend to determine
+      your observing locations this way, this module is not needed.
 EOD
-    [ [ qw{ LWP::UserAgent LWP::Protocol URI } ] => <<'EOD' ],
+    __all( qw{ LWP::UserAgent LWP::Protocol URI } => <<'EOD' ),
       These modules are required if you want to use URLs in the init(),
       load(), or source() methods. If you do not intend to use URLs
       there, you do not need these packages. All three packages are
@@ -68,7 +74,7 @@ EOD
       so you may get them implicitly if you install some of the other
       optional modules.
 EOD
-	$is_5_012 ? () : [ 'Time::y2038' => <<'EOD' .
+	$is_5_012 ? () : __any( 'Time::y2038' => <<'EOD' .
       This module is not required, but if installed allows you to do
       computations for times outside the usual range of system epoch to
       system epoch + 0x7FFFFFFF seconds.
@@ -86,7 +92,7 @@ EOD
       so-called 'usual range.' Time::y2038 will be used, though, if it
       is available.
 EOD
-    ],
+    ),
 );
 
 sub make_optional_modules_tests {
@@ -147,35 +153,20 @@ EOD
 }
 
 sub optionals {
-    my @rslt;
-    foreach my $spec ( @optionals ) {
-	my $module = $spec->[0];
-	if ( 'ARRAY' eq ref $module ) {
-	    push @rslt, @{ $module };
-	} elsif ( ref $module ) {
-	    confess 'Module spec may not be a ', ref $module, ' reference';
-	} else {
-	    push @rslt, $module;
-	}
-    }
-    return @rslt;
+    return ( map { $_->modules() } @optionals );
 }
 
 sub recommend {
     my $need_some;
-    SPEC_LOOP:
-    foreach my $spec ( @optionals ) {
-	my ( $module, $message ) = @{ $spec };
-	foreach my $mod ( ref $module ? @{ $module } : $module ) {
-	    eval "require $mod; 1"
-		and next SPEC_LOOP;
-	}
+    foreach my $mod ( @optionals ) {
+	defined( my $msg = $mod->recommend() )
+	    or next;
 	$need_some++
 	    or warn <<'EOD';
 
-The following optional modules were not found:
+The following optional modules were not available:
 EOD
-	warn format_module_line( $module ), $message;
+	warn "\n$msg";
     }
     $need_some
 	and warn <<'EOD';
@@ -188,27 +179,19 @@ EOD
     return;
 }
 
-sub format_module_line {
-    my ( $module ) = @_;
-    ref $module
-	or return "\n    * $module is not installed.\n";
-    return "\n    * None of " . join( ', ', @{ $module } ) .
-	" is installed.\n";
-}
-
 1;
 
 __END__
 
 =head1 NAME
 
-Astro::Coord::ECI::Recommend - Recommend modules to install. 
+My::Module::Recommend - Recommend modules to install. 
 
 =head1 SYNOPSIS
 
  use lib qw{ inc };
- use Astro::Coord::ECI::Recommend;
- Astro::Coord::ECI::Recommend->recommend();
+ use My::Module::Recommend;
+ My::Module::Recommend->recommend();
 
 =head1 DETAILS
 
@@ -246,7 +229,7 @@ This static method simply returns the names of the optional modules.
 
 =head2 recommend
 
- Astro::Coord::ECI::Recommend->recommend();
+ My::Module::Recommend->recommend();
 
 This static method examines the current Perl to see which optional
 modules are installed. If any are not installed, a message is printed to
